/* 
 * Process Variable Library with JVisu Socket Server
 * 
 * Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
 *           (C) 2005 Robert Schwebel <r.schwebel@pengutronix.de>
 *           (C) 2005 Sascha Hauer <s.hauer@pengutronix.de>
 *		
 * PURPOSE
 *
 * This is a backing store for using 8Bit (FRAM, SRAM) devices on a
 * Processor which can't do 8Bit accesses.
 *
 * At the moment we use hardcoded memory adddresses and sizes, we should
 * make this configurable. Also we use /dev/mem for mapping the device
 * into memory, so we need root privileges
 * 
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <pv.h>

int pv_8bit_bs_init(pv_backing_store_t *store, const char *filename_unused);
static int pv_8bit_bs_exit(pv_backing_store_t *store);
static int pv_8bit_bs_read(pv_backing_store_t *store, off_t, size_t len);
static int pv_8bit_bs_write(pv_backing_store_t *store, off_t, size_t len);

struct pv_8bit_bs_priv {
	int fd;
	void *mem;
};

int pv_8bit_bs_init(pv_backing_store_t *store, const char *filename) {
	struct pv_8bit_bs_priv *priv;
	int ret = 0;

	printf("initialising 8Bit backing store\n");

	priv = malloc(sizeof(*priv));
	if(!priv) {
		perror("malloc");
		ret = -1;
		goto exit;
	}

	memset(priv, 0, sizeof(priv));

	store->priv = priv;
	store->exit = pv_8bit_bs_exit;
	store->read = pv_8bit_bs_read;
	store->write = pv_8bit_bs_write;

	ret = open(filename, O_RDWR | O_SYNC);
	if(ret < 0) {
		perror("open");
		goto exit_free;
	}
	priv->fd = ret;

	priv->mem = mmap(NULL, store->size, PROT_READ | PROT_WRITE, 
		         MAP_SHARED, priv->fd, 0);
	if(!priv->mem) {
		perror("mmap");
		ret = -1;
		goto exit_close;
	}

	return 0;

 exit_close:
	close(priv->fd);
 exit_free:
	free(priv);
 exit:
	return ret;
}

static int pv_8bit_bs_exit(pv_backing_store_t *store) {
	struct pv_8bit_bs_priv *priv = store->priv;

	munmap(priv->mem, store->size);
	close(priv->fd);
	return 0;
}

static int pv_8bit_bs_read(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_8bit_bs_priv *priv = store->priv;
	unsigned long *src = priv->mem + (offset << 2);
	unsigned char *dst = store->map + offset;
	int i;

	VFPRINTF(2, stderr, "%s: off: %d len: %d\n",__FUNCTION__,(int)offset,len);

	for(i=0; i<len;i++)
		*dst++ = *src++;

	return 0;
}

static int pv_8bit_bs_write(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_8bit_bs_priv *priv = store->priv;
	unsigned char *src = store->map + offset;
	unsigned long *dst = priv->mem + (offset << 2);	
	int i;

	VFPRINTF(2, stderr, "%s: off: %d len: %d\n",__FUNCTION__,(int)offset,len);

	for(i=0; i<len;i++)
		*dst++ = *src++;

	return 0;
}
